/*
     Modbus RTU Master    NL.
*/

#define F_CPU 14745600UL 																						//    
	
#include "NC8D2R2O.h"																							// 

#include <avr/io.h>																								//  -
#include <avr/eeprom.h>																							//    EEPROM

#define COM_RX_BUFFER_SIZE 32																					//   COM 
#define COM_TX_BUFFER_SIZE 32																					//   COM 

#define COM1_PROTOCOL MODBUS																					//   COM1  Modbus RTU ( - 1  )
#define COM2_PROTOCOL MODBUS																					//   COM2  Modbus RTU ( - 3  )

#include "COM.h"																								//   COM 

#define MAX_SLAVE_DEVICE 2																						//  SLAVE 
#define TIME_OUT_3DPAS 10																						// -    (1   10 )
#define COUNT_3DPAS_MAX 3																						//    NL-3DPAS,       

//------------------------------------------  EEPROM ----------------------------------------------
unsigned char EE_Address_device EEMEM=0x01;																		//   EEPROM
unsigned char EE_COM_Speed EEMEM=0x06;																			//   EEPROM

unsigned char EE_Name[] EEMEM="NC8D4R";																			//   EEPROM
unsigned char EE_Version[] EEMEM="270318";																		//   EEPROM

/*
          
 8       Modbus RTU

  4     (0- ; 1 NL-16DI; 2 NL-16DO; 3 NL-8R; 4 NL-8AI; 5 NL-3DPAS)

 4     .         (          ).
    ,      .

:
0x0252

    Modbus RTU 0x02
  NL-3DPAS (5 )
  2

       ,        ,      
*/
int EE_DeviceConfig[MAX_SLAVE_DEVICE] EEMEM={0x0151,0x0252};													// 

//-------------------------------------------  RAM ------------------------------------------------
char Address_device;																							// 
char COM_Speed;																									// 
char Name[10];																									// 
char Version[6];																								// 

char Func;																										// 
int SubFunc;																									// 
int Data;																										// 

char buf1[COM_RX_BUFFER_SIZE];																					//      COM1
char buf2[COM_RX_BUFFER_SIZE];																					//      COM2
int Address_EEP;																								//       EEPROM

char Time_Out_3DPAS=0;																							//  -     3DPAS
int CountTimeOut_3DPAS=0;																						// -     3DPAS
char TimeOut_3DPAS_EN=0;																						//  -     3DPAS

unsigned char DeviceCount=0;																					//   
int DeviceConfig[MAX_SLAVE_DEVICE];																				//  
char Count_Response_Slave[MAX_SLAVE_DEVICE]={0,0};																//     
int Status_Word=0;																								//    
	
int AddrReg;																									//     
char QuestFunc;																									//     
int AmountReg;																									// / /
char Response_Ok;																								//    

int Parameter[6];																								//     

//--------------------------------------------------------------------------------------------------------------
//===================================     0 =======================================
//--------------------------------------------------------------------------------------------------------------
SIGNAL(TIMER0_COMP_vect)																						//    -     
{
	if (TimeOut_3DPAS_EN)
	{
		if (CountTimeOut_3DPAS>=TIME_OUT_3DPAS)
		{
			Time_Out_3DPAS=1;																					//   -    3DPAS
			TimeOut_3DPAS_EN=0;																					// -     3DPAS
		}
		else CountTimeOut_3DPAS++;																				//  -    3DPAS
	}
}

//========================================    ========================================
void Init(void)
{
	int temp;																									// 
	long Speed;																									//   
	
	Address_device=eeprom_read_byte(&EE_Address_device);														//    EEPROM  

	COM_Speed=eeprom_read_byte(&EE_COM_Speed);																	//   EEPROM
	
	switch (COM_Speed)
	{
		case 0x03: Speed=1200; break;
		case 0x04: Speed=2400; break;
		case 0x05: Speed=4800; break;
		case 0x06: Speed=9600; break;
		case 0x07: Speed=19200; break;
		case 0x08: Speed=38400; break;
		case 0x09: Speed=57600; break;
		case 0x0A: Speed=115200; break;
		default: Speed=9600;
	}
	
	COM_Init(COM_PORT1,Speed);																					//  COM1 (Slave   )
	COM_Init(COM_PORT2,9600);																					//  COM2 (Master  NL-3DPAS)
	
	Address_EEP=(int)&EE_Name;																					//   EEPROM    
	for ( temp=0; temp<10; temp++) Name[temp]=eeprom_read_byte(Address_EEP+(uint8_t*)temp);						//    EEPROM
	
	Address_EEP=(int)&EE_Version;																				//   EEPROM    
	for (temp=0; temp<6; temp++) Version[temp]=eeprom_read_byte(Address_EEP+(uint8_t*)temp);					//  
		
	Address_EEP=(int)&EE_DeviceConfig;																			//   EEPROM   
	for (temp=0; temp<MAX_SLAVE_DEVICE; temp++) DeviceConfig[temp]=eeprom_read_word((uint16_t*)Address_EEP+temp);//   
	
	TIMSK|=(1<<OCIE0);																							//     0 (  -   )
	OCR0=144;																									//   0,   - (  10 )
	TCCR0=0x0F;																									//  - (  , .  1024)
	
	sei();																										//  
	
	_delay_ms(1000);																							//   ,  Slave   
}

//===========================      COM  ============================
void RTU1()
{
/*
	buf1[0]				-   
	buf1[1]				-  
	buf1[2]				-  
	buf1[3]+buf1[4]		-   ( )
	buf1[5]+buf1[6]		-  (  0x05, 0x06  ,  0x01-0x04   )
	buf1[7]+buf1[8]		- CRC  (   Modbus RTU)
*/
	char temp;																									// 
	int int_temp;																								// 
	
	if (Read_Modbus(COM_PORT1,buf1)==0x01)																		//    ,        (.  )
	{
		if (buf1[1]==Address_device) 																			//  
		{
			Func=buf1[2];																						//  
			switch(Func)																						//  
			{
//==========================================    ===========================================
			case 0x03:
				SubFunc=(buf1[3]<<8)|buf1[4];																	//  
				Data=(buf1[5]<<8)|buf1[6];																		// 
			
				switch (SubFunc)																				//  
				{
//----------------------------------  , ,  -----------------------------------
/*
Parameter[0]-  NL-3DPAS   001
Parameter[1]-  NL-3DPAS   001
Parameter[2]-  NL-3DPAS   001
Parameter[3]-  NL-3DPAS   002
Parameter[4]-  NL-3DPAS   002
Parameter[5]-  NL-3DPAS   002
*/
				case 0x0000 ... 0x0005:
					if ((Data+SubFunc>0x0006) || (Data==0))														//       
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}

					for (int_temp=0; int_temp<Data; int_temp++)													//    EEPROM
					{
						buf1[int_temp*2]=Parameter[SubFunc+int_temp-0x0000]>>8;
						buf1[int_temp*2+1]=Parameter[SubFunc+int_temp-0x0000] & 0x00FF;
					}
					buf1[int_temp*2]=0;

					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,Data);
					break;

//--------------------------------------------    ---------------------------------------------
				case 0x00C8:
					if (Data!=4)																				//      
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}
				
					for (int_temp=0; int_temp<8; int_temp++) buf1[int_temp]=Name[int_temp];						//  
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,4);
					break;
				
//------------------------------------------    -------------------------------------------
				case 0x00D4:
					if (Data!=3)																				//  -    
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}
				
					for (int_temp=0; int_temp<6; int_temp++)													//  
					buf1[int_temp]=Version[int_temp];
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,3);
					break;

//----------------------------------       -----------------------------------
				case 0x0100:
					if (Data!=1)																				//  -    
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}

					buf1[0]=Status_Word>>8;
					buf1[1]=Status_Word & 0x00FF;
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,1);
					break;
				
//-----------------------------------------------   ------------------------------------------------
				case 0x0200:
					if (Data!=1)																				//  -    
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}

					temp=eeprom_read_byte(&EE_Address_device);													//    EEPROM  
					buf1[0]=0;
					buf1[1]=temp;
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,1);
					break;
				
//----------------------------------------------   -----------------------------------------------
				case 0x0201:
					if (Data!=1)																				//  -    
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}
				
					temp=eeprom_read_byte(&EE_COM_Speed);														//    EEPROM  
					buf1[0]=0;
					buf1[1]=temp;
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,1);
					break;
				
//--------------------------------------     --------------------------------------
				default:												
					buf1[0]=0x02;																				// ""
					Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);
				}
				break;

//=========================================    =========================================
			case 0x06:																							//    (0x06)
				SubFunc=(buf1[3]<<8)|buf1[4];																	//  
				Data=(buf1[5]<<8)|buf1[6];																		// 
			
				switch (SubFunc)																				//      
				{
//-----------------------------------------------   ------------------------------------------------
				case 0x0200:
					if ((Data==0) || (Data>0xF7))																// -     
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}
				
					eeprom_write_byte(&EE_Address_device,Data);													//  EEPROM  
					buf1[0]=Data>>8;
					buf1[1]=Data & 0x00FF;
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,1);
					break;
				
//-------------------------------------------    --------------------------------------------
				case 0x0201:
					if ((Data<0x03) || (Data>0x0A))																//       
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}
				
					eeprom_write_byte(&EE_COM_Speed,Data);														//  EEPROM   
					buf1[0]=Data>>8;
					buf1[1]=Data & 0x00FF;
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,1);
					break;
				
//--------------------------------------     --------------------------------------
				default: 																						//    
					buf1[0]=0x02;
					Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);					// ""
				}
				break;

//==========================================    ===========================================
			case 0x10:																							//     (0x10)
				SubFunc=(buf1[3]<<8)|buf1[4];																	//  
				int_temp=(buf1[5]<<8)|buf1[6];																	// - 

				switch (SubFunc)																				//      
				{
//--------------------------------------------    ---------------------------------------------
				case 0x00C8:
					if (int_temp!=4)																			//     
					{
						buf1[0]=0x03;
						Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);				// ""
						break;
					}
				
					Address_EEP=(int)&EE_Name;																	//   EEPROM    
	
					for (int_temp=0; int_temp<8; int_temp++)													// 
					{
						Name[int_temp]=buf1[int_temp+8];
						eeprom_write_byte(Address_EEP+(uint8_t*)int_temp,Name[int_temp]);						//  EEPROM     
					}
					Write_Response_Modbus(COM_PORT1,Address_device,Func,SubFunc,buf1,5);
					break;
				
//--------------------------------------     --------------------------------------
				default: 																						//    
					buf1[0]=0x02;
					Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);					// ""
				}
				break;
			
//---------------------------------------     ----------------------------------------
			default:
				buf1[0]=0x01;
				Write_Response_Modbus(COM_PORT1,Address_device,Func | 0x80,SubFunc,buf1,1);						// ""
			}
		}
	}
}

//--------------------------------------------------------------------------------------------------------------
//=====================================      ======================================
//--------------------------------------------------------------------------------------------------------------
char WaitResponse(void)
{
	Time_Out_3DPAS=0;																							//   -     3DPAS
	CountTimeOut_3DPAS=0;																						//  -     3DPAS
	TimeOut_3DPAS_EN=1;																							//  -     3DPAS
	
	while (1)																									//          -
	{
		RTU1();																									//       
		if (Read_Modbus(COM_PORT2,buf2)==1) return 1;															//      
		else if (Time_Out_3DPAS==1) return 0;																	//  -
	}
}

//--------------------------------------------------------------------------------------------------------------
//=======================================    NL-3DPAS =======================================
//--------------------------------------------------------------------------------------------------------------
void NL3DPAS(void)
{
	QuestFunc=0x03;																								//  
	AddrReg=0x0000;																								//  
	AmountReg=1;																								//         (    buf2)
	buf2[0]=0; buf2[1]=3;																						//    

	Write_Request_Modbus(COM_PORT2,DeviceConfig[DeviceCount]>>8,QuestFunc,AddrReg,buf2,AmountReg);				//    ( COM , , ,  , ,  )

	if (WaitResponse()==1)																						//  
	{
		Response_Ok=1;																							//      
		
		if (buf2[1]!=DeviceConfig[DeviceCount]>>8) Response_Ok=0;												//  
		if (buf2[2]!=QuestFunc) Response_Ok=0;																	//  
		if (buf2[3]!=6) Response_Ok=0;																			//  
		
		Parameter[DeviceCount*3]=((int)buf2[4]<<8)|buf2[5];														//  
		Parameter[DeviceCount*3+1]=((int)buf2[6]<<8)|buf2[7];													//  
		Parameter[DeviceCount*3+2]=((int)buf2[8]<<8)|buf2[9];													//  
	}
	else Response_Ok=0;																							//     
	
	if (Response_Ok==1)
	{
		Status_Word|=(1<<DeviceCount);																			//     
		Count_Response_Slave[DeviceCount]=0;
	}
	else
	{
		if (Count_Response_Slave[DeviceCount]>=COUNT_3DPAS_MAX) Status_Word&=~(1<<DeviceCount);					//     
		else Count_Response_Slave[DeviceCount]++;
	}
}

//=============================================   =============================================
int main(void)
{
	Init();
	
	while(1)																									// 
	{
		RTU1();																									//       

		switch((DeviceConfig[DeviceCount]>>4) & 0x000F)															//   (0x01 - NL-16DI, 0x02 - NL-16DO, 0x03 - NL-8R, 0x04 - NL-8AI, 0x05 - NL-3DPAS)
		{
			case 0x00:				break;																		// 
			case 0x05: NL3DPAS();	break;																		//   NL-3DPAS
		}
		
		if (DeviceCount>=MAX_SLAVE_DEVICE) DeviceCount=0;														//   ,   
		else DeviceCount++;																						//,    
		
		_delay_ms(10);																							//   10 
	}
}
